#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

#include "lyn.h"

int cddb_sum(int n)
{
  char buf[12],*p;
  int ret=0;

  /* For backward compatibility this algorithm must not change */
  sprintf(buf, "%lu", n);
  for (p = buf; *p != '\0'; p++)
    ret += (*p - '0');
  return (ret);
}

unsigned long cddb_discid(int tot_trks)
{
  int i,i1,t=0,n=0;
  int min0,sec0,min1,sec1;

  /* For backward compatibility this algorithm must not change */
  for (i=0; i<tot_trks; i++) {
    i1=thiscd.trk[i].start/75;
    sec0=i1 % 60;
    min0=(i1-sec0)/60;
    i1=thiscd.trk[i+1].start/75;
    sec1=i1 % 60;
    min1=(i1-sec1)/60;
    n += cddb_sum((min0 * 60) + sec0);
    t += ((min1 * 60) + sec1) - ((min0 * 60) + sec0);
  }
  return ((n % 0xff) << 24 | t << 8 | tot_trks);
}

char *gen_cddb_id(void)
{
  sprintf(thiscd.cddb_id,"%08x",cddb_discid(thiscd.ntracks));
  return thiscd.cddb_id;
}

void log_cddb_debug(char *fmt, ...)
{
#if DEBUGLEVEL>1
  FILE *f;
  va_list ap;
  char s[4096];

  va_start(ap,fmt);
  vsprintf(s,fmt,ap);
  va_end(ap);
  if (!strlen(s))
    return;
  f=fopen("lyn-cddb-debug","a");
  if (!f)
    return;
  fprintf(f,"%d: %s\n",getpid(),s);
  fclose(f);
#endif
}

#define DEFAULT_CDDB_SERVER "cddb.sonic.net"

int read_cddb_dbase(char *id, struct cdinfo *cdd)
{
  int i;
  char *s,*ss;
  
  if (!config.cddbservers || !config.cddbservers[0])
    return read_cddb_dbase_s(id,cdd,DEFAULT_CDDB_SERVER);
  s=(char *)malloc(strlen(config.cddbservers));
  if (!s) {
    perror("malloc");
    return -1;
  }
  strcpy(s,config.cddbservers);
  ss=strtok(s," ");
  while (ss) {
    i=read_cddb_dbase_s(id,cdd,ss);
    if (i==0) {
      free(s);
      return 0;
    }
    if (i<0)
      continue;
    if (i>0) {
      free(s);
      return i;
    }
    ss=strtok(NULL," ");
  }
  free(s);
  return -1;
}

int read_cddb_dbase_s(char *id, struct cdinfo *cdd, char *server)
{
  int i,i1,endread,sc,nc,sofar;
  char s[80],*ss;
  char aid[10],category[10];
  char localhost[256],buf[8192],username[80];
  struct hostent *hp;
  struct sockaddr_in name;
  
  report_status("READ CDDB");
  if (id)
    strcpy(aid,id);
  else if (thiscd.cddb_id[0])
    strcpy(aid,thiscd.cddb_id);
  else
    strcpy(aid,gen_cddb_id());
  if (!aid[0])
    return -1;
  if (gethostname(localhost,sizeof(localhost))) {
    perror("gethostname");
    return -2;
  }
  report_status("DB LOOKUP");
  if ((hp=gethostbyname(server))==NULL) {
    fprintf(stderr,"unknown host: %s\n",server);
    return -3;
  }
  report_status("SOCKET");
  if ((sc=socket(AF_INET,SOCK_STREAM,0))<0) {
    perror("socket");
    return -4;
  }
  memset(&name,0,sizeof(struct sockaddr_in));
  name.sin_family=AF_INET;
  name.sin_port=htons(888);
  memcpy(&name.sin_addr,hp->h_addr_list[0],hp->h_length);
  report_status("CONNECTING");
  if (connect(sc,(struct sockaddr *)&name,sizeof(struct sockaddr_in))<0) {
    perror("connect");
    return -5;
  }
  report_status("CONNECTED");
  log_cddb_debug("connected ok");
  if (recv(sc,buf,sizeof(buf),0)<=0) {
    send(sc,"quit\n",5,0);
    perror("recv");
    return -7;
  }
  log_cddb_debug("recv: %s",buf);
  if (buf[0]>='4') {
    if (atoi(buf)==432)
      report_status("DB DENIED");
    else if (atoi(buf)==433)
      report_status("DB FULL");
    else if (atoi(buf)==434)
      report_status("DB LOAD");
    else {
      sprintf(s,"DB ERR %d",atoi(buf));
      report_status(s);
    }
    send(sc,"quit\n",5,0);
    close(sc);
    return 1;
  }
  report_status("DB HELLO");  
  strcpy(username,(char *)cuserid(NULL));
  if (!username[0])
    strcpy(username,"unknownusername");
  sprintf(buf,"cddb hello %s %s lyn %s\n",username,localhost,bare_version);
  log_cddb_debug("send: %s",buf);
  if (send(sc,buf,strlen(buf),0)<0) {
    perror("send");
    send(sc,"quit\n",5,0);    
    close(sc);
    return -6;
  }
  if (recv(sc,buf,sizeof(buf),0)<=0) {
    perror("recv");
    send(sc,"quit\n",5,0);    
    close(sc);
    return -7;
  }
  log_cddb_debug("recv: %s",buf);
  if (buf[0]>='4') {
    if (atoi(buf)==431) {
      report_status("HSK ERR");
      send(sc,"quit\n",5,0);      
      close(sc);
      return -2;
    } else if (atoi(buf)==402)
      report_status("HSK ALR");
    else {
      sprintf(s,"DB ERR %d",atoi(buf));
      report_status(s);
      send(sc,"quit\n",5,0);      
      close(sc);
      return -10;
    }
  }
  report_status("DB QUERY");
  sprintf(buf,"cddb query %s %d",aid,thiscd.ntracks);
  for (i=0; i<thiscd.ntracks; i++) {
    sprintf(s," %d",thiscd.trk[i].start);
    strcat(buf,s);
  }
  sprintf(s," %d\n",thiscd.length);
  strcat(buf,s);
  log_cddb_debug("send: %s",buf);
  if (send(sc,buf,strlen(buf),0)<0) {
    perror("send");
    send(sc,"quit\n",5,0);
    close(sc);
    return -6;
  }  
  if (recv(sc,buf,sizeof(buf),0)<=0) {
    perror("recv");
    send(sc,"quit\n",5,0);
    close(sc);
    return -7;
  }
  log_cddb_debug("recv: %s",buf);  
  if (buf[0]=='2') {
    if (atoi(buf)==200)
      report_status("QUERY MATCH");
    else if (atoi(buf)==202) {
      report_status("NO MATCH");
      send(sc,"quit\n",5,0);
      close(sc);
      return 9;
    } else if (atoi(buf)==211) {
      report_status("FUZZ MATCH");
      /* for the moment, give up without the exact match */
      send(sc,"quit\n",5,0);
      close(sc);
      return 10;
    }
  } else if (buf[0]>='4') {
    if (atoi(buf)==403)
      report_status("DB CORRUPT");
    else if (atoi(buf)==409)
      report_status("NO HSK");
    else {
      sprintf(s,"DB ERR %d",atoi(buf));
      report_status(s);
    }
    send(sc,"quit\n",5,0);
    close(sc);
    return -11;
  }
  for (i=0; i<strlen(buf) && !isspace(buf[i]); i++);
  for (; i<strlen(buf) && isspace(buf[i]); i++);
  for (i1=0; i<strlen(buf) && !isspace(buf[i]); i++,i1++)
    category[i1]=buf[i];
  category[i1]=0;
  for (; i<strlen(buf) && isspace(buf[i]); i++);
  for (i1=0; i<strlen(buf) && !isspace(buf[i]); i++,i1++)
    aid[i1]=buf[i];
  aid[i1]=0;
  sprintf(buf,"cddb read %s %s\n",category,aid);
  log_cddb_debug("send: %s",buf);
  if (send(sc,buf,strlen(buf),0)<0) {
    perror("send");
    send(sc,"quit\n",5,0);
    close(sc);
    return -6;
  }
  if ((sofar=recv(sc,buf,sizeof(buf),0))<=0) {
    perror("recv");
    send(sc,"quit\n",5,0);
    close(sc);
    return -7;
  }
  while (buf[0]=='2' && sofar>0 && 
         (buf[sofar-3]!='\n' && buf[sofar-2]!='.' && buf[sofar-3]!='\n')) {
    if ((i=recv(sc,buf+sofar,1,0))<=0) {
      perror("recv");
      send(sc,"quit\n",5,0);
      close(sc);
      return -7;
    }
    sofar+=i;
  }
  log_cddb_debug("recv: %s",buf);
  if (buf[0]=='2') {
    if (atoi(buf)==210)
      report_status("REC MATCH");
  } else if (buf[0]>='4') {
    if (atoi(buf)==401) {
      report_status("NO ENTRY");
      send(sc,"quit\n",5,0);
      close(sc);
      return 11;
    } else if (atoi(buf)==402)
      report_status("SRVR ERR");
    else if (atoi(buf)==403)
      report_status("DB CORRUPT");
    else if (atoi(buf)==409)
      report_status("NO HSK");
    else {
      sprintf(s,"DB ERR %d",atoi(buf));
      report_status(s);
    }
    send(sc,"quit\n",5,0);
    close(sc);
    return -11;
  }
  for (i=0; i<strlen(buf); i++)
    if (buf[i]=='\r')
      buf[i]='\n';
  ss=strtok(buf,"\n");
  do {
    if ((ss=strtok(NULL,"\n"))==NULL)
      continue;
    log_cddb_debug("buffered: %s",ss);
    if (ss[0]=='#' || !ss[0])
      continue;
    if (ss[0]=='.')
      break;
    if (!strncmp("DISCID",ss,6))
      ;
    else if (!strncmp("DTITLE",ss,6)) {
      strcpy(cdd->title,&ss[7]);
      for (i=0; i<strlen(cdd->title); i++)
        if (cdd->title[i]==' ' &&
            cdd->title[i+1]=='/' &&
            cdd->title[i+2]==' ') {
          cdd->title[i]=':';
          cdd->title[i+1]=' ';
          for (i1=i+2; i1<strlen(cdd->title); i1++)
            cdd->title[i1]=cdd->title[i1+1];
          break;
        }
    } else if (!strncmp("TTITLE",ss,6)) {
      if (!isdigit(ss[6])) {
        report_status("BAD TTITLE");
        continue;
      }
      strcpy(cdd->songtitle[atoi(&ss[6])],strchr(ss,'=')+1);
    } else if (!strncmp("EXTD",ss,4))
      /* we don't care (yet) */;
    else if (!strncmp("PLAYORDER",ss,9))
      /* be nice to incorporate this when the shuffling code is ready */;
  } while (ss);
  send(sc,"quit\n",5,0);
  log_cddb_debug("send: quit\n");
  close(sc);
  log_cddb_debug("connection closed");
  return 0;
}

