#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
#include <ncurses/curses.h>

#include "lyn.h"

int write_local_dbase(int id, struct cdinfo *cdd)
{
  char s[256];
  int i,f;

  sprintf(s,"%s%.8s.cd",config.cddbpath,cdd->cid);
  f=open(s,O_WRONLY | O_CREAT | O_TRUNC,0666);
  if (f<0)
    return -1;
  sprintf(s,"%-43s # disc ID\n",cdd->cid);
  write(f,&s,strlen(s));
  sprintf(s,"%-43s # disc title\n",cdd->title);
  write(f,&s,strlen(s));
 /* sprintf(s,"%-43s # disc lyrics filename\n",cdd->lyrics_fn); */
  sprintf(s,"%-43s # disc CDDB id\n",thiscd.cddb_id);
  write(f,&s,strlen(s));
  for (i=0; i<cdd->ntracks; i++) {
    sprintf(s,"%-43s # track %d title\n",cdd->songtitle[i],i+1);
    write(f,&s,strlen(s));
  }
  strcpy(s,"! END\n");
  write(f,&s,strlen(s));
  sprintf(s,"  %s",version_string);
  write(f,&s,strlen(s));
  close(f);
  return 0;
}

#define CLEAN_STRING()   for (i=strlen(s)-1; i>0; i--) \
    if (s[i]=='#' && s[i-1]!='\\') { \
      s[i]=0; \
      break;\
    } else if (s[i]=='#' && s[i-1]=='\\') { \
      strncpy(&s[i-1],&s[i],strlen(&s[i])); \
    } else if (s[i]=='\r' || s[i]=='\n')\
      s[i]=0;\
  i=strlen(s)-1;\
  while (s[i--]==32 && i>=0)\
    s[i+1]=0;\
  if (s[strlen(s)-1]==32)\
    s[strlen(s)-1]=0


int read_local_dbase(int id, struct cdinfo *cdd)
{
  char s[512];
  int i,i1,i2;
  FILE *f;
  
  report_status("LOCAL DB");
  sprintf(s,"%s%d.cd",config.cddbpath,id);
  f=fopen(s,"rt");
  if (!f) {
    dwrite(19,14,13,"none");
    return -1;
  }
  dwrite(19,14,13,"local");
  fgets(s,81,f);
  cdd->nid=strtol(s,0,10);
  fgets(s,81,f);
  CLEAN_STRING();
  if (!s[0])
    sprintf(cdd->title,"<no title; disc id %s>",id);
  else
    strcpy(cdd->title,s);
  fgets(s,81,f);
  if (s[0]!=32 && s[0]!='#' && s[0]!='\r' && s[0]=='\n') {
    CLEAN_STRING();
    strcpy(cdd->cddb_id,s);
  } 
  i2=0;
  while (!feof(f)) {
    fgets(s,511,f);
    if (s[0]=='!')
      break;
    CLEAN_STRING();
    strcpy(cdd->songtitle[i2++],s);
  }
  fclose(f);
  return 0;
}

void edit_songdata(void)
{
  int i,done=0;
  char *s;

  s=(char *)malloc(512);
  if (!s) {
    perror("edit_songdata: malloc:");
    return;
  }
  report_status("EDITING");
  echo();
  nodelay(stdscr,FALSE);
  i=0;
  highlight_track(0,cur_track);
  while (!done) {
    if (!i) {
      dwrite(39,5,15,charstr(42,32));
      gotoxy(39,5);
      getstr(s);
      if (*s)
        strcpy(thiscd.title,s);
      dwrite(39,5,15,"%-42.42s",thiscd.title);
      if (!thiscd.title[0])
        done=1;
      i++;
    } else if (i>0 && i<=tracklist_bottom) {
      dwrite(30,6+i,31,"%2d:",i);
      gotoxy(41,6+i);
      dwrite(41,6+i,15,charstr(40,32));
      gotoxy(41,6+i);
      getstr(s);
      if (*s)
        if (!strcmp(s," "))
          thiscd.songtitle[i-1][0]=0;
        else
          strcpy(thiscd.songtitle[i-1],s);
      dwrite(41,7+i-1,7,"%-40.40s",thiscd.songtitle[i-1]);
      dwrite(30,7+i-1,7,"%2d:",i);
      i++;
    } else
      done=1;
  }
  free(s);
  nodelay(stdscr,TRUE);
  move(0,0);
  noecho();
}


int read_dbase(void)
{
  int i,done;
  char *s,*ss;

  thiscd.nid=atoi(thiscd.cid);
  s=(char *)malloc(strlen(config.dborder));
  strcpy(s,config.dborder);
  ss=strtok(s," ");
  done=0;
  while (ss && !done) {
    if (!strcasecmp(ss,"local")) {
      if (!read_local_dbase(thiscd.nid,&thiscd))
        done=1;
#ifdef SQL_DBASE
    } else if (!strcasecmp(ss,"sql")) {
      if (!sql_lookup_cd(thiscd.nid,&thiscd))
        done=2;
#endif
#ifdef CDDB_DBASE
    } else if (!strcasecmp(ss,"cddb")) {
      if (!read_cddb_dbase(thiscd.cddb_id,&thiscd))
        done=3;
#endif
    }
    ss=strtok(NULL," ");
  }
  free(s);
  if (!done) {
    dwrite(19,14,13,"none ");
    blank_tracknames();
    thiscd.nid=0;
    return 1;
  }
  report_status("DB LOADED");
  update_topline();
  draw_tracklist();
  cd_status();
/*  if (!dd.tracks)
    dd.tracks=thiscd.ntracks;*/
  if (done==1) {
    dwrite(19,14,13,"local");
    return 0;
  } else if (done==2) {
    dwrite(19,14,13,"sql  ");
    if (config.sqlcopytolocal) {
      write_local_dbase(thiscd.nid,&thiscd);
      report_status("DB COPIED");
    }
    return 0;
  } else if (done==3) {
    dwrite(19,14,13,"cddb ");
    if (config.cddbcopytolocal) {
      write_local_dbase(thiscd.nid,&thiscd);
      report_status("DB COPIED");
    }
    return 0;
  }
  return -1;

#ifdef SQL_DBASE_OBS
  thiscd.nid = atoi(thiscd.cid);
  report_status("QUERYING DB");
  i=sql_lookup_cd(thiscd.nid,&thiscd);
  if (!i) {
    report_status("DB LOADED");
    dwrite(19,14,13,"net  ");
    /* if the lookup worked, dd is set and everything's fine,
     * so be done with it.  If not, then go through with the
     * usual local lookup.
     *
     * should be done in other order, but this way for testing.
     */
    return 0;
  }
  report_status("NO DB MATCH");
#endif
#if 0
  update_topline();
  draw_tracklist();
  return read_local_dbase(thiscd.nid,&thiscd);
#endif
}

int write_dbase(void)
{
  int i;
  
#ifdef SQL_DBASE_SUBMIT
  report_status("QUERYING DB");
  i=sql_submit_cd(thiscd.nid,&thiscd);
  if (!i) {
    report_status("DB SENT");
    return 0;
  }
#endif
  return write_local_dbase(thiscd.nid,&thiscd);

}

int blank_tracknames(void)
{
  int i;
  
  sprintf(thiscd.title,"<no dbase entry; disc id %s>",thiscd.cid);
  for (i=0; i<100; i++)
    sprintf(thiscd.songtitle[i],"Track %d",i+1);  
}

int blank_trackdata(void)
{
  thiscd.nid=0;
  thiscd.ntracks=0;
  blank_tracknames();
  return 0;
}
