/*
 * SQL remote database functions for Lyn
 *
 * mostly a gratuitous kludging-together of something I've
 * never seen done before.  Every cdplayer does disc-recognition,
 * stores song names, etc.  xmcd has a network database for it.
 * Now so does lyn, but lyn also gets the lyrics.  :)
 *
 * First coding: Thu May 22 02:37:19 PDT 1997, aqua
 */

#ifdef SQL_DBASE

#include <stdio.h>
#include <msql.h>
#include <stdlib.h>
#include <string.h>
#include <ncurses/curses.h>

#include "lyn.h"

char *msqlEscape(const char *str)
/* Stolen from the code to mSQL's w3-msql module.  Curiously, mSQL's C API
 * doesn't provide an SQL escape function.
 */
{
  char *new,*cp1,*cp2;
                                                
  new = (char *)malloc(2 * strlen(str));
  cp1 = (char *)str;
  cp2 = new;
  while (*cp1) {
    if (*cp1 == '\\') {
      *cp2++ = *cp1++;
    } else if (*cp1=='\'' || *cp1=='[' || *cp1==']' ||
               *cp1=='^' || *cp1=='$') {
      *cp2++ = '\\';
    }
    *cp2++ = *cp1++;
  }
  *cp2 = 0;
  return(new);
}

int sql_lookup_cd(int id, struct cdinfo *cdd)
{
  int i,i1,dbh;
  char query[256];
  m_result *rsp;
  m_row r;

  /* Connect to the server & database */
  
  report_status("CONNECTING");
  dbh=msqlConnect(SQL_DBASE_SERVER);
  if (dbh<0) {
#if DEBUGLEVEL>2
    dwrite(1,LINES-1,10,msqlErrMsg);
#endif
    log_sql_debug("connect failed:");
    log_sql_debug(msqlErrMsg);
    report_status("NO CONNECT");
    return dbh;
  }
  report_status("ACCESSING");
  log_sql_debug("selectdb");
  if ((i=msqlSelectDB(dbh,"lt"))<0) {
#if DEBUGLEVEL>2
    dwrite(1,LINES-1,10,msqlErrMsg);
#endif
    log_sql_debug(msqlErrMsg);
    report_status("DB MISSING");
    msqlClose(dbh);
    return i;
  }

  /* Get the disc title/artist stuff */

  sprintf(query,"select distinct id1,title,artist from titles " \
                " where id1='%d'",id);
  log_sql_debug("query:");
  log_sql_debug(query);
  report_status("QUERY TITLE");
  if ((i=msqlQuery(dbh,query))<=0) {
#if DEBUGLEVEL>2
#if 0
    /* For some odd reason, the following segfaults on about the ninth
     * call.  No other call to dwrite() does, nor any of the debugging
     * calls with query, or sprintf() with id.  Hmm.
     */
    dwrite(1,LINES-1,10,"No record for id %d (%s)",id,query);
#endif
#endif
    log_sql_debug("query failed: %d row(s)",i);
    log_sql_debug(msqlErrMsg);
    report_status("NO MATCH");
    msqlClose(dbh);
    return -1;
  }
  report_status("PARSING");
  rsp=msqlStoreResult();  
  log_sql_debug("fetchrow");
  if ((r=msqlFetchRow(rsp))==NULL) {
#if DEBUGLEVEL>2
    dwrite(1,LINES-1,10,msqlErrMsg);
#endif
    log_sql_debug("fetchrow failed:");
    log_sql_debug(msqlErrMsg);
    report_status("!ROW ERROR");
    return -1;
  }
  log_sql_debug("fetchrow ok, got:");
  sprintf(cdd->title,"%s: %s",r[2],r[1]);
  log_sql_debug(cdd->title);
  msqlFreeResult(rsp);


  /* Go get the songs */
  log_sql_debug("fetching songs");
  report_status("QUERY SONGS");
  sprintf(query,"select distinct id, title, track from songs " \
                " where id='%d' order by track",id);
  log_sql_debug("query:");
  log_sql_debug(query);                
  if ((i=msqlQuery(dbh,query))<=0) {
#if DEBUGLEVEL>2
    dwrite(1,LINES-1,10,"No songs for id %d",id);
#endif
    log_sql_debug("query failed: %d row(s)",i);
    log_sql_debug(msqlErrMsg);
    report_status("NO SONGS");
    for (i1=0; i1<100; i1++)
      sprintf(cdd->songtitle[i1],"Track %d",i1+1);
    log_sql_debug("no songs; disconnecting");
    msqlClose(dbh);
    return i;
  }
  rsp=msqlStoreResult();
  report_status("PARSING");
/*  cdd->tracks=0; */
  while ((r=msqlFetchRow(rsp))!=NULL)  {
    strcpy(cdd->songtitle[atoi(r[2])-1],r[1]);
/*    cdd->tracks++; */
    log_sql_debug("song:");
    log_sql_debug(r[1]);
  }
  log_sql_debug("end of songs");
/*  cdd->lyrics_fn[0]=0; */
#if DEBUGLEVEL>2
  dwrite(1,LINES-1,10,"DB ok");
#endif
  report_status("DB OK");
  msqlFreeResult(rsp);
  log_sql_debug("closing connect");
  msqlClose(dbh);
  return 0;
}



void log_sql_debug(char *fmt, ...)
{
#if DEBUGLEVEL>1
  FILE *f;
  va_list ap;
  char s[1024];

  va_start(ap,fmt);
  vsprintf(s,fmt,ap);
  va_end(ap);
  if (!strlen(s))
    return;
  f=fopen("lyn-sql-debug","a");
  if (!f)
    return;
  fprintf(f,"%d: %s\n",getpid(),s);
  fclose(f);
#endif
}



int sql_submit_cd(const int id, const struct cdinfo *cdd)
{
  int i,i1,dbh;
  char s[80];
  char query[512],wtitle[256];
  char *artist, *title;
  char *eartist,*etitle;
  m_result *rsp;
  m_row r;

  strcpy(wtitle,cdd->title);
#if DEBUGLEVEL>2
  dwrite(1,LINES-1,10,"submit(%d)",id);
#endif

  /* Connect to the server & database */
  
  log_sql_debug("beginning submit");
  report_status("CONNECTING");
  dbh=msqlConnect(SQL_DBASE_SERVER);
  if (dbh<0) {
#if DEBUGLEVEL>2
    dwrite(1,LINES-1,10,msqlErrMsg);
#endif
    report_status("NO CONNECT");
    log_sql_debug("failed connect:");
    log_sql_debug(msqlErrMsg);
    return dbh;
  }
  log_sql_debug("connect ok");
  /* new submissions don't go directly to the database,
   * but rather to an intermediary database where new
   * entries are queued (and checked, in a modest effort to
   * keep people from fiddling the dbase).  'ldb' is
   * not publicly-writable.
   *
   * This is done for convenience only; if any problems
   * arise, it will be scrapped and replaced with something
   * more directly intervening, like xmcd's email approach.
   */
  report_status("ACCESSING");
  if ((i=msqlSelectDB(dbh,"ldbi"))<0) {
#if DEBUGLEVEL>2
    dwrite(1,LINES-1,10,msqlErrMsg);
#endif
    report_status("DB MISSING");
    log_sql_debug("failed selectdb:");
    log_sql_debug(msqlErrMsg);
    msqlClose(dbh);
    return i;
  }
  log_sql_debug("selectdb ok");
  
  /* See if it's already there */
  
  report_status("QUERY");
  sprintf(query,"select distinct id1,title,artist from titles " \
                " where id1='%d'",id);
  log_sql_debug("query:");
  log_sql_debug(query);
  i=msqlQuery(dbh,query);

  if (i>1) {
    /* if we matched more than one record, the dbase is probably
     * fiddled, so give up rather than possibly exacerbate it.
     */
#if DEBUGLEVEL>2
    dwrite(1,LINES-1,10,"Multiple records exist");
#endif
    report_status("MULTI");
    log_sql_debug("multiple records matched");
    msqlClose(dbh);
    return -2;
  }
  log_sql_debug("query<=0, ok");
/*  artist = (char *)malloc(strlen(cdd->title));
  title = (char *)malloc(strlen(cdd->title));*/
  strcpy(wtitle,cdd->title);
  artist=strtok(wtitle,":");
  title=strtok(NULL,":")+1;
  eartist=msqlEscape(artist);
  etitle=msqlEscape(title);
  if (i==1) {
    /* If a record already exists */
    report_status("MATCHED");
    log_sql_debug("1 record exists");
    sprintf(query,"update titles set title='%s', artist='%s' where id1='%d'",
                    etitle,eartist,id);
  } else if (i==0) {
    /* If this is a new record */
    report_status("NEW");
    log_sql_debug("new record");
/*    sprintf(query,"insert into titles ( id1, id2, title, artist, supp ) " \
                  "values ( '%d', NULL, '%s', '%s', NULL )",id,
                    msqlEscape(title),msqlEscape(artist));
 */
    sprintf(query,"insert into titles values ( '%d', '', '%s', '%s', '' )",
                  id,etitle,eartist);
  } else {
    report_status("DB ERR");
    log_sql_debug("wierdness:");
    log_sql_debug(msqlErrMsg);
    /* something wierd happened */
#if DEBUGLEVEL>2
    dwrite(1,LINES-1,10,msqlErrMsg);
#endif
    msqlClose(dbh);
    free(eartist);
    free(etitle);
    return i;
  }
  free(eartist);
  free(etitle);
  log_sql_debug("query:");
  log_sql_debug(query);
  if ((i=msqlQuery(dbh,query))<=0) {
#if DEBUGLEVEL>2
    dwrite(1,LINES-1,10,"Error in submit/update query");
#endif
    report_status("DB ERR");
    log_sql_debug("failed query");
    log_sql_debug(msqlErrMsg);
    msqlClose(dbh);
    log_sql_debug("connect closed");
    return -1;
  }
  log_sql_debug("query ok");
  report_status("SEND TITLE");
  sprintf(query,"delete from songs where id='%d'",id);
  log_sql_debug("query:");
  log_sql_debug(query);
  msqlQuery(dbh,query);
  report_status("SEND SONGS");
  log_sql_debug("send songs: %d tracks",thiscd.ntracks);
  for (i1=0; i1<thiscd.ntracks; i1++) {
    sprintf(s,"SEND SONG %d",i1+1);
    report_status(s);
    if (!cdd->songtitle[i1])
      continue;
    etitle=msqlEscape(cdd->songtitle[i1]);
    sprintf(query,"insert into songs ( id, title, track ) " \
                  "values ( '%d', '%s', %d )",
                    id,etitle,i1+1);
    free(etitle);
    log_sql_debug("query:");
    log_sql_debug(query);
    if (i=msqlQuery(dbh,query)<=0) {
      log_sql_debug("query failed: %d row(s)",i);
      log_sql_debug(msqlErrMsg);
      msqlClose(dbh);
      return i;
    }
  }
  log_sql_debug("song-queries ok");
  log_sql_debug("sql close");
  msqlClose(dbh);
  return 0;
}

#endif